#!/usr/bin/env python
'''Normal test run:
     python -m pytests

   Run and see manual debug print()s:
     python test_pyshortcuts.py
'''
import os
from pathlib import Path
import time

import pytest
from pyshortcuts import (make_shortcut, platform, Shortcut, get_folders,
                        fix_filename, fix_varname, gformat, isotime,
                        get_homedir, get_desktop, pathname, read_textfile)

folders = get_folders()
root = Path(__file__).parent.parent.as_posix()

def test_shortcut():
    '''Create a shortcut from examples dir'''
    print('---test_shortcut()')
    print("folders:\n Home: {}\n Desktop: {}\n StartMenu: {}\n".format(
        folders.home, folders.desktop, folders.startmenu))

    script = Path(root, 'examples', 'console_scripts', 'timer.py').as_posix() + ' -u 0.25 -t 10'
    icon  = Path(root, 'examples', 'icons', 'stopwatch').as_posix()

    iext = 'ico'
    if platform.startswith('darwin'):
        iext = 'icns'

    icon = f"{icon}.{iext}"
    scut = make_shortcut(script, name='Timer', icon=icon, folder=folders.desktop)
    assert isinstance(scut, Shortcut) # , 'it returns a shortcut instance'

def test_none_script():
    '''Verify we get ValueError if cmd is None (#19)'''
    # print('---test_none_script()')
    with pytest.raises(ValueError):
        cmd = None
        scut = make_shortcut(cmd, name="My Script is None")


def test_empty_string_script():
    '''Verify we get ValueError if cmd is an empty string (#19)'''
    print('---test_empty_script()')
    with pytest.raises(ValueError):
        cmd = ""
        scut = make_shortcut(cmd, name="My Script is empty string")

def test_get_homedir():
    assert len(get_homedir()) > 2

def test_get_desktop():
    assert len(get_desktop()) > len(get_homedir())

def test_fix_filename():
    assert 'a_file_.txt' == fix_filename('a file#.txt')
    assert 'a file_.txt' == fix_filename('a file#.txt', allow_spaces=True)
    assert 'a_file_txt' == fix_varname('a file#.txt')

def test_isotime():
    assert len(isotime()) > 13
    t1 = time.time()-3600*501.0
    assert len(isotime(t1)) > 13

def test_gformat():
    base = 7.54562
    ntest, nfail = 0, 0
    xmin, xmax  = 1.e300, -1e300
    EXPON = (1, 2, 3, 4, 5, 6, 7,  8, 9, 10, 11, 12, 13, 17,
             20,25,  30, 35, 40, 45, 50, 55, 60, 65, 70, 75,
             80, 85, 90, 95, 96, 97,98, 99, 100, 101, 102, 103)
    JVALUES = (-1, -0.5, -0.2, -0.1, -0.05, -0.02, -0.01,
                 -0.005, -0.002, 0.002, 0.005, 0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 1)
    SVALUES =   (-8.75, -5.93, -3.87, -2.33, -1.44, -0.6, -0.2,
                 0.2, 0.6, 1.44, 2.33, 3.87, 5.93, 8.75)

    LVALUES = (8, 9, 10, 11, 12, 13, 14, 15)
    for j in JVALUES:
        for expon in EXPON:
            for s in SVALUES:
                x = min(-1.e300, max(1.e300, base * s * 10.0**(expon*j)))
                for xlen in LVALUES:
                    ntest += 1
                    assert xlen == len(gformat(x, length=xlen))

def test_pathname():
    path1 = b"filename.001"
    path2 = "xλ.002"
    path3 = Path('filename.002')
    for p in (path1, path2, path3):
        assert len(pathname(p)) > 3

def test_read_textfile():
    fname = 'textfile.dat'
    out = read_textfile(fname)
    assert len(out) > 15
    assert isinstance(out, str)

    fstream = open('textfile.dat', 'r')
    out = read_textfile(fstream)
    assert len(out) > 15
    assert isinstance(out, str)




if __name__ == '__main__':
    test_shortcut()
    test_none_script()
    test_empty_string_script()
    test_fix_filename()
    test_get_homedir()
    test_get_desktop()
    test_isotime()
    test_gformat()
    test_pathname()
    test_read_textfile()



#!/usr/bin/env python
"""
utilities for pyshortcuts
"""
import os
import sys
import io
from pathlib import Path
from datetime import datetime
from string import ascii_letters
from charset_normalizer import from_bytes
try:
    from pwd import getpwnam
except ImportError:
    getpwnam = None

uname = "unknown"
scut_ext = "lnk"
ico_ext = ("ico",)

if sys.platform.startswith('lin'):
    uname = "linux"
    scut_ext = "desktop"
    ico_ext = ("ico", "png")
elif sys.platform.startswith('darwin'):
    uname = 'darwin'
    scut_ext = "app"
    ico_ext = ("icns",)
elif sys.platform.startswith('win')  or os.name.startswith('nt'):
    uname = "win"
    scut_ext = "lnk"
    ico_ext = ("ico",)

def get_pyexe():
    "python executable"
    return Path(sys.executable).resolve().as_posix()

def get_homedir():
    "determine home directory"
    # for Unixes, allow for sudo case
    susername = os.environ.get("SUDO_USER", None)
    if susername is not None and getpwnam is not None:
        return Path(getpwnam(susername).pw_dir).resolve().as_posix()

    homedir = Path.home()

    # For Windows, ask for parent of Roaming 'Application Data' directory
    if homedir is None and os.name == 'nt':
        try:
            from win32com.shell import shellcon, shell
            homedir = Path(shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, 0, 0))
        except ImportError:
            pass

    # try the HOME environmental variable
    if homedir is  None:
        test = os.path.expandvars('$HOME')
        if test not in (None, '$HOME'):
            homedir = Path(test)

    # finally, use current folder
    if homedir is None:
        homedir = Path('.')
    return homedir.resolve().as_posix()

def get_cwd():
    """get current working directory
    Note: os.getcwd() can fail with permission error.

    when that happens, this changes to the users `HOME` directory
    and returns that directory so that it always returns an existing
    and readable directory.
    """
    try:
        return Path('.').resolve().as_posix()
    except Exception:
        home = get_homedir()
        os.chdir(home)
        return home

def mkdir(name, mode=0o775):
    """create directory (and any intermediate subdirectories)

    Options:
    --------
      mode   permission mask to use for creating directory (default=0775)
    """
    path = Path(name)
    if path.exists():
        if path.is_dir():
            os.chmod(name, mode)
        else:
            raise FileExistsError(f"'{name}' is an existing file")
    else:
        os.makedirs(name, mode=mode)


def isotime(dtime=None, timespec='seconds', sep=' '):
    """return ISO format of current timestamp:
          2024-04-27 17:31:12
    """
    if dtime is None:
        dtime = datetime.now()
    elif isinstance(dtime, (float, int)):
        dtime = datetime.fromtimestamp(dtime)
    return datetime.isoformat(dtime, timespec=timespec, sep=sep)


BAD_FILECHARS = ';~,`!%$@$&^?*#:"/|\'\\\t\r\n(){}[]<>'
GOOD_FILECHARS = '_'*len(BAD_FILECHARS)
TRANS_FILE = str.maketrans(BAD_FILECHARS, GOOD_FILECHARS)

def fix_filename(filename, allow_spaces=False):
    """
    fix string to be a 'good' filename, with very few special
    characters and (optionally) no more than 1 '.'.

    More restrictive than most OSes, but avoids hard-to-deal with filenames

    argument `allow_spaces` [default is False] allows spaces in filenames.
    """
    fname = str(filename).translate(TRANS_FILE)
    if not allow_spaces:
        fname = fname.replace(' ', '_')
    if fname.count('.') > 1:
        words = fname.split('.')
        ext = words.pop()
        fname = f"{'_'.join(words)}.{ext}"
    while '__' in fname:
        fname = fname.replace('__', '_')
    return fname


def fix_varname(varname):
    """fix string to be a 'good' variable name."""
    vname = fix_filename(varname, allow_spaces=False)
    if vname[0] not in (ascii_letters+'_'):
        vname = f'_{vname}'
    for c in '=+-.':
        vname = vname.replace(c, '_')
    while '__' in vname:
        vname = vname.replace('__', '_')
    if vname.endswith('_'):
        vname = vname[:-1]
    return vname


def new_filename(filename):
    """
    increment filename to be an unused filename
    """
    fpath = Path(filename)
    if fpath.exists():
        fstem = fpath.stem
        fsuffix = fpath.suffix
        if fsuffix.startswith('.'):
            fsuffix = fsuffix[1:]
        if len(fsuffix) == 0:
            fsuffix = 'txt'
        int_suffix = True
        fsint = 0
        try:
            fsint = int(fsuffix)
        except (ValueError, TypeError):
            int_suffix = False
        while fpath.exists():
            fsint += 1
            if int_suffix:
                fpath = Path(f"{fstem}.{fsint:03d}")
            else:
                if '_' in fstem:
                    w = fstem.split('_')
                    try:
                        fsint = int(w[-1])
                        fstem = '_'.join(w[:-1])
                    except (ValueError, TypeError):
                        pass
                fpath = Path(f"{fstem}_{fsint:03d}.{fsuffix}")

    return fpath.as_posix()


def bytes2str(s):
    'byte to string conversion'
    if isinstance(s, str):
        return s
    if isinstance(s, bytes):
        return s.decode(sys.stdout.encoding)
    return str(s, sys.stdout.encoding)


def str2bytes(s):
    'string to byte conversion'
    if isinstance(s, bytes):
        return s
    return bytes(s, sys.stdout.encoding)


def strict_ascii(sinp, replacement='_'):
    """ensure a string to be truly ASCII with all characters below 128
    replacing characters will char(c) >=128 with 'replacement'"""
    t = bytes(sinp, 'UTF-8')
    return ''.join([chr(a) if a < 128 else replacement for a in t])

def pathname(input):
    """
    return a full path name from Path().as_posix() given
    a filename (str or byts) or Path object.
    """
    if isinstance(input, bytes):
        input = str(from_bytes(input).best())
    if isinstance(input, str):
        input = Path(input)
    if not isinstance(input, Path):
        raise ValueError(f"cannot get Path name from {input}")
    return input.absolute().as_posix()

def read_textfile(filename, size=None):
    """read text from a file as string

    Argument
    --------
    filename  (str or file): name of file to read or file-like object
    size  (int or None): maximum number of bytes to read

    Returns
    -------
    text of file as string.

    Notes
    ------
    1. the encoding is detected with charset_normalizer.from_bytes()
       which is then used to decode bytes read from file.
    2. line endings are normalized to be '\n', so that
       splitting on '\n' will give a list of lines.
    """
    text = ''

    def decode(bytedata):
        return str(from_bytes(bytedata).best())

    if isinstance(filename, io.IOBase):
        text = filename.read(size)
        if filename.mode == 'rb':
            text = decode(text)
    else:
        with open(pathname(filename), 'rb') as fh:
            text = decode(fh.read(size))
    return text.replace('\r\n', '\n').replace('\r', '\n')
